"""Climax configuration file."""

# !/usr/bin/env python
# -*- coding: utf-8 -*-

# builtins
import datetime

# externals
import numpy as np
import torch.nn as nn

# locals
from pysegcnn.core.models import SegNet, USegNet
from climax.core.constants import PREDICTANDS
from climax.core.models import Huang2020

# -----------------------------------------------------------------------------
# ERA5 downscaling configuration ----------------------------------------------
# -----------------------------------------------------------------------------

# ERA5 predictor variables on pressure levels
# ERA5_P_PREDICTORS = ERA5_P_VARIABLES

# # ERA5 predictor variables on single levels
# ERA5_S_PREDICTORS = ERA5_S_VARIABLES

# # ERA5 predictor variables
# ERA5_PREDICTORS = ERA5_VARIABLES

# ERA5 pressure levels
ERA5_PLEVELS = [500, 850]

# include day of year as predictor
DOY = True

# -----------------------------------------------------------------------------
# Observations ----------------------------------------------------------------
# -----------------------------------------------------------------------------

# target variable: check if target variable is valid
PREDICTAND = 'tas'
assert PREDICTAND in PREDICTANDS

# -----------------------------------------------------------------------------
# Calibration and validation period -------------------------------------------
# -----------------------------------------------------------------------------

# calibration period: training and validation
CALIB_PERIOD = np.arange(
    datetime.datetime.strptime('1981-01-01', '%Y-%m-%d').date(),
    datetime.datetime.strptime('1991-01-01', '%Y-%m-%d').date())

# validation period: testing
VALID_PERIOD = np.arange(
    datetime.datetime.strptime('1991-01-01', '%Y-%m-%d').date(),
    datetime.datetime.strptime('2011-01-01', '%Y-%m-%d').date())

# -----------------------------------------------------------------------------
# Model training configuration ------------------------------------------------
# -----------------------------------------------------------------------------

# network architecture
NET = USegNet

# number of convolutional layers and corresponding filter sizes
FILTERS = [32, 64, 128, 256]

# loss function to optimize
# for temperature: nn.L1Loss or nn.MSELoss (NLL of normal distribution)
# for precipitation: to implement
LOSS = nn.MSELoss()

# whether to randomly shuffle time steps or to conserve time series for model
# training
SHUFFLE = True

# whether to normalize the training data to [0, 1] (True) or to standardize to
# mean=0, std=1 (False)
NORM = True

# batch size: number of time steps processed by the net in each iteration
BATCH_SIZE = 16

# learning rate
LR = 0.001

# regularization rate
LAMBDA = 0

# network training configuration
TRAIN_CONFIG = {
    'checkpoint_state': {},
    'epochs': 250,
    'save': True,
    'save_loaders': False,
    'early_stop': True,
    'patience': 25,
    'multi_gpu': True,
    'classification': False
    }

# whether to overwrite existing models
OVERWRITE = False

# split reference period into NYEARS intervals
# this limits the amount of data that is loaded into memory during prediction
# NOTE: if NYEARS=None, load entire reference period into memory at once
NYEARS = 5
